﻿// Accord Math Library
// The Accord.NET Framework
// http://accord-framework.net
//
// Copyright © César Souza, 2009-2014
// cesarsouza at gmail.com
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

namespace Accord.Math
{
    using System.Runtime.CompilerServices;
    using Accord.Math.Optimization;

    /// <summary>
    ///   Mathematics, including <see cref="Matrix">matrix algebra</see>, 
    ///   <see cref="Accord.Math.Decompositions">matrix decompositions</see>
    ///   and <see cref="Accord.Math.Optimization">numeric optimization</see>.
    /// </summary>
    /// 
    /// <remarks>
    /// <para>
    ///   This namespace contains a <see cref="Matrix">matrix extension library</see>,
    ///   along with a suite of numerical 
    ///   <see cref="Accord.Math.Decompositions">matrix decomposition methods</see>, 
    ///   <see cref="Accord.Math.Optimization">optimization algorithms</see> for 
    ///   <see cref="AugmentedLagrangian">constrained</see> and
    ///   <see cref="BroydenFletcherGoldfarbShanno">unconstrained problems</see>,
    ///   special functions and other <see cref="Accord.Math.Tools">tools</see> for 
    ///   scientific applications.</para> 
    ///   
    /// <para>
    ///   Importing the Accord.Math namespace automatically brings support for the
    ///   Extension Matrix Library, which extends common .NET data types with several
    ///   mathematics-related extension methods.</para>
    ///   
    /// <para>
    ///   The namespace class diagram is shown below. </para>
    ///   <img src="..\diagrams\classes\Accord.Math.png" />
    ///   
    /// <para>
    ///   The namespace class diagram for the optimization namespace is shown below. </para>
    ///   <img src="..\diagrams\classes\Accord.Math.Optimization.png" />
    ///   
    /// <para>
    ///   The namespace class diagram for the matrix decompositions is shown below. </para>
    ///   <img src="..\diagrams\classes\Accord.Math.Decompositions.png" />
    ///   
    /// <para>
    ///   Please note that class diagrams dedicated for matrix decompositions
    ///   are also available at <see cref="Accord.Math.Decompositions"/> page.
    /// </para>
    /// </remarks>
    /// 
    /// <seealso cref="Accord.Math.Comparers"/>
    /// <seealso cref="Accord.Math.ComplexExtensions"/>
    /// <seealso cref="Accord.Math.Decompositions"/>
    /// <seealso cref="Accord.Math.Differentiation"/>
    /// <seealso cref="Accord.Math.Environments"/>
    /// <seealso cref="Accord.Math.Geometry"/>
    /// <seealso cref="Accord.Math.Integration"/>
    /// <seealso cref="Accord.Math.Kinematics"/>
    /// <seealso cref="Accord.Math.Optimization"/>
    /// <seealso cref="Accord.Math.Wavelets"/>
    /// 
    [CompilerGenerated]
    class NamespaceDoc
    {
    }
}
